@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright 2021-2023 Advanced Micro Devices, Inc.
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -43,10 +43,10 @@ namespace rocwmma
     namespace MatrixLayout
     {
         /**
-         * \ingroup rocwmma
-         * \defgroup Matrix Layouts
+         * \defgroup Matrix_Layouts Matrix Layouts
          *
          * @brief Definition and metadata on supported matrix layouts.
+         * @{
          *
          * These layouts are based in matrix coordinate space. They map each of the wavefront lanes
          * into corresponding (X , Y) = (row,  col) coordinates for a particular memory layout.
@@ -62,11 +62,6 @@ namespace rocwmma
          *   E.g. Sum of incremental offsets for [i = 0, iteration)
          */
 
-        /**
-         * \ingroup Matrix Layouts
-         * @{
-         */
-
         /**
          * ColNT and RowNT layouts: The MFMA alignments. Matrix core instructions
          * have specific mapping for inputs and outputs: inputs being contiguous
@@ -185,7 +180,6 @@ namespace rocwmma
          *       ...     |   ...    |
          *      RegN*S   |  (N, S)  |
          *       ...     |   ...    |
-         * @}
          */
 
         template <uint32_t BlockDim,
@@ -208,16 +202,15 @@ namespace rocwmma
                 // elements in both row_major or col_major data layouts.
                 // This layout cannot support for VW > 1 in col_major data layout otherwise the
                 // ordering is broken.
-                static_assert(!(std::is_same<DataLayout, col_major>::value && VectorWidth > 1),
+                static_assert(!(std::is_same_v<DataLayout, col_major> && VectorWidth > 1),
                               "ColNT in col_major does not support VectorWidth > 1");
+
+                // Must ensure that MaxVectorWidth fits inside the leading dimension
+                static_assert(std::is_same_v<DataLayout, row_major> && (MaxVectorWidth <= BlockK),
+                    "MaxVectorWidth is larger than BlockK dimension. Try reducing MaxVectorWidth");
             };
         };
 
-        /**
-         * \ingroup Matrix Layouts
-         * @{
-         */
-
         /**
          * RowNT Layout
          *
@@ -315,7 +308,6 @@ namespace rocwmma
          *       ...     |   ...    |
          *      RegN*S   |  (N, S)  |
          *       ...     |   ...    |
-         * @}
          */
         template <uint32_t BlockDim,
                   uint32_t BlockK,
@@ -337,15 +329,15 @@ namespace rocwmma
                 // elements in both in row_major or col_major data layouts.
                 // This layout cannot support for VW > 1 in row_major data layout otherwise the
                 // ordering is broken.
-                static_assert(!(std::is_same<DataLayout, row_major>::value && VectorWidth > 1),
+                static_assert(!(std::is_same_v<DataLayout, row_major> && VectorWidth > 1),
                               "RowNT in row_major does not support VectorWidth > 1");
+
+                // Must ensure that MaxVectorWidth fits inside the leading dimension
+                static_assert(std::is_same_v<DataLayout, col_major> && (MaxVectorWidth <= BlockK),
+                    "MaxVectorWidth is larger than BlockK dimension. Try reducing MaxVectorWidth");
             };
         };
 
-        /**
-         * \ingroup Matrix Layouts
-         * @{
-         */
         /**
          * Col and Row layouts: unrestricted vector width col and row ordering.
          * Unline their NT cousins, these layouts do not guarantee the same
@@ -391,7 +383,7 @@ namespace rocwmma
          * The column ordering however will be determined as a multiple of MaxVW to
          * fulfill MaxVW priority.
          *
-         *  Data Layout: row_major <same as ColNT>
+         *  Data Layout: row_major \<same as ColNT\>
          *  Register Mapping (BlockDim < 64):
          *
          *      N = Max VectorWidth
@@ -424,7 +416,7 @@ namespace rocwmma
          * across multiple registers. Col ordering still fufills MaxVW
          * priority first.
          *
-         *  Data Layout: row_major <same as ColNT>
+         *  Data Layout: row_major \<same as ColNT\>
          *  Register Mapping (BlockDim == 64):
          *
          *      N = Max Vector Width
@@ -457,7 +449,7 @@ namespace rocwmma
          * to satisfy MaxVW priority before moving on to the next BlockDim
          * segment.
          *
-         *  Data Layout: row_major <same as ColNT>
+         *  Data Layout: row_major \<same as ColNT>
          *  Register Mapping (BlockDim > 64):
          *
          *      Priority 1: Visit MaxVW Segments
@@ -498,7 +490,6 @@ namespace rocwmma
          *   Reg2N-1 | (0, 1, N-1) | (0, 1, 2N-1) | ... | (1, 1, N-1) | ... | (N-1, 1, 64-1)   |
          *   ....    |  ...        |    ...       | ... |   ....      | ... |     ...          |
          *
-         * @}
         */
 
         template <uint32_t BlockDim,
@@ -519,11 +510,6 @@ namespace rocwmma
             };
         };
 
-        /**
-         * \ingroup Matrix Layouts
-         * @{
-         */
-
         /**
          * Row Layout
          *
@@ -564,7 +550,7 @@ namespace rocwmma
          * The row ordering however will be determined as a multiple of MaxVW to
          * fulfill MaxVW priority.
          *
-         *  Data Layout: col_major <same as RowNT>
+         *  Data Layout: col_major \<same as RowNT\>
          *  Register Mapping (BlockDim < 64):
          *
          *      N = Max VectorWidth
@@ -597,7 +583,7 @@ namespace rocwmma
          * across multiple registers. Row ordering still fufills MaxVW
          * priority first.
          *
-         *  Data Layout: col_major <same as RowNT>
+         *  Data Layout: col_major \<same as RowNT\>
          *  Register Mapping (BlockDim == 64):
          *
          *      N = Max Vector Width
@@ -631,7 +617,7 @@ namespace rocwmma
          * to satisfy MaxVW priority before moving on to the next BlockDim
          * segment.
          *
-         *  Data Layout: col_major <same as RowNT>
+         *  Data Layout: col_major \<same as RowNT\>
          *  Register Mapping (BlockDim > 64):
          *
          *      Priority 1: Visit MaxVW Segments
@@ -671,7 +657,6 @@ namespace rocwmma
          *   ....    |  ...        |    ...       | ... |   ....      | ... |     ...          |
          *   Reg2N-1 | (0, 1, N-1) | (0, 1, 2N-1) | ... | (1, 1, N-1) | ... | (N-1, 1, 64-1)   |
          *   ....    |  ...        |    ...       | ... |   ....      | ... |     ...          |
-         * @}
          */
         template <uint32_t BlockDim,
                   uint32_t BlockK,
@@ -693,6 +678,7 @@ namespace rocwmma
 
         template <typename LayoutT>
         using OrthogonalLayout_t = typename detail::OrthogonalLayout<LayoutT>::Type;
+        /** @}*/
 
     } // namespace MatrixLayout
 
