@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright 2021-2023 Advanced Micro Devices, Inc.
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -439,54 +439,54 @@ namespace rocwmma
             * that VW will be mapped orthogonally to the column.
             * This pattern considers VW up to MaxVW, BlockDim <= 64 and BlockDim > 64.
             *
-            * Iterative thread offset cycles: Fill MaxVW => Fill BlockDim => Fill K
+            * Iterative thread stride cycles (same for all threads):
+            *   Fill MaxVW => Fill BlockK => Fill BlockDim
             *
-            * Index on VW segments first, BlockDimSegs second. Below shows the indexing
-            * order of columns for two full major cycles:
+            * Example:
+            *  BlockDim = 128   BlockK = 16
+            *  MaxVW = 4       VW = 1
             *
-            * E.g.
-            * WaveSize = 64    Iterations = 8
-            * BlockDim = 128   BlockK = 8          BlockDimSegs = 2
-            * VectorWidth = 2  MaxVectorWidth = 4  VWSegs = 2
+            *  BlockDim Stride Count = 2, BlockDimStride = (64, 0)
+            *  BlockK   Stride Count = 4, BlockKStride   = (0,  4)
+            *  VW       Stride Count = 4, VWStride       = (0,  1)
             *
-            * Minor cycle = VWSegs = 2 iterations
-            * Major cycle = VWSegs * BlockDimSegs = 4 iterations
-            *
-            * iteration offsets:
-            * i0 = (0, 0)   i1 = (0, 2)  i2 = (64, 0) i3 = (64, 2)
-            * i4 = (0, 4)   i5 = (0, 6)  i6 = (64, 4) i7 = (64, 6)
+            *  Stride mapping (BlockDim, BlockK, VW)
+            *  C_n = Matrix column
+            *  i_n = cumulative iteration
             *
             *   kDim --------->
-            *
-            *   i0          i1          i4          i5
-            *   v_____ _____v_____ _____v_____ _____v_____ _____
-            *   |     |     |     |     |     |     |     |     |
-            *   |     |     |     |     |     |     |     |     |
-            *   | C0  |  C1 |  C2 |  C3 |  C8 |  C9 | C10 | C11 | ...
-            *   |     |     |     |     |     |     |     |     |
-            *   |_____|_____|_____|_____|_____|_____|_____|_____|
-            *   i2          i3          i6          i7
-            *   v_____ _____v_____ _____v_____ _____v_____ _____
-            *   |     |     |     |     |     |     |     |     |
-            *   |     |     |     |     |     |     |     |     |
-            *   | C4  |  C5 |  C6 |  C7 | C12 | C13 | C14 | C15 | ...
-            *   |     |     |     |     |     |     |     |     |
-            *   |_____|_____|_____|_____|_____|_____|_____|_____|
-            *   ^(128, 0)                                       ^(BlockDim, BlockK)
+            *                     VW Stride
+            *   BlockDim          |--1--|
+            *   |                 |-- BlockK Stride = 4 --|
+            *   |                 i0(0,0,0)   i2(0,0,2)   i4(0,1,0)   i6(0,1,2)         i14(0,3,2)
+            *   |            --   v_____ _____v_____ _____v_____ _____v_____ _____      v_____  _____
+            *   v            |    |     |     |     |     |     |     |     |     |     |     ||     |
+            *                |    |     |     |     |     |     |     |     |     |     |     ||     |
+            *       BlockDim 64   | C0  |  C1 |  C2 |  C3 |  C4 |  C5 | C6  | C7  | ... | C14 || C15 |
+            *        Stride  |    |     |     |     |     |     |     |     |     |     |     ||     |
+            *                --   |_____|_____|_____|_____|_____|_____|_____|_____|     |_____||_____|
+            *                     i16(1,0,0)  i18(1,0,2)  i20(1,1,0)  i22(1,1,2)        i30(1,3,2)
+            *                     v_____ _____v_____ _____v_____ _____v_____ _____      v_____  _____
+            *                     |     |     |     |     |     |     |     |     |     |     ||     |
+            *                     |     |     |     |     |     |     |     |     |     |     ||     |
+            *                     | C0  |  C1 |  C2 |  C3 | C4  | C5  | C6  | C7  | ... | C14 || C15 |
+            *                     |     |     |     |     |     |     |     |     |     |     ||     |
+            *                     |_____|_____|_____|_____|_____|_____|_____|_____|     |_____||_____|
+            *                     ^(128, 0)                                                           ^(BlockDim, BlockK)
             *   ...                                          ...
             *
             * Register file (for all VectorWidths = [1, MaxVectorWidth]):
             *
-            * Elements 0......64
-            *          ______
-            *  Reg0    |  C0  |
-            *  Reg1    |  C1  |
-            *  Reg2    |  C2  |
-            *  Reg3    |  C3  |
-            *  Reg4    |  C4  |
-            *  Reg5    |  C5  |
+            * Elements 0..............63
+            *           ______________
+            *  Reg0    |  C0 [63:0]    |
+            *  Reg1    |  C1 [63:0]    |
+            *  Reg2    |  C2 [63:0]    |
             *  ...       ...
-            *  Reg15   |  C15 |
+            *  Reg15   |  C15[63:0]    |
+            *  Reg16   |  C0 [127:64]  |
+            *  ...       ...
+            *  Reg31   |  C15 [127:64] |
             }*/
 
             template <uint32_t BlockDim,
@@ -505,16 +505,28 @@ namespace rocwmma
                         WaveSize = IOTraits::ThreadsPerIO,
 
                         // Number of BlockDim columns gathered per cycle of MaxVW
-                        MaxKPerIO = WaveSize * MaxVectorWidth / BlockDim,
+                        MaxKPerIO = WaveSize * MaxVectorWidth / std::min(BlockDim, WaveSize),
+
+                        BlockDimStride_X = WaveSize,
+                        BlockDimStride_Y = 0u,
+
+                        BlockKStride_X = 0u,
+                        BlockKStride_Y = MaxKPerIO,
+
+                        VWStride_X = 0u,
+                        VWStride_Y = VectorWidth,
 
                         // Flag for large BlockDim
                         LargeDim = BlockDim >= WaveSize,
 
-                        // Number of column segments (> 0 if LargeDim )
-                        BlockDimSegs = BlockDim / WaveSize,
+                        // Number of segments in BlockDim direction
+                        BlockDimSegs = std::max(BlockDim / BlockDimStride_X, 1u),
 
-                        // Number of vector width segments
-                        VWSegs = MaxVectorWidth / VectorWidth,
+                        // Number of segments in the BlockK direction
+                        BlockKSegs = BlockK / BlockKStride_Y,
+
+                        // Number of segments in the MaxVW direction
+                        VWSegs = MaxVectorWidth / VWStride_Y,
 
                         // Number of columns per wave (> 0 if !LargeDim)
                         WaveSegs = WaveSize / BlockDim,
@@ -536,7 +548,7 @@ namespace rocwmma
                 ROCWMMA_DEVICE static inline typename Traits::MatrixCoordT baseOffset()
                 {
                     // TODO: Use constexpr if on C++17
-                    if(Traits::LargeDim)
+                    if constexpr(Traits::LargeDim)
                     {
                         return make_coord2d(threadIdx.x % Traits::WaveSize, 0u);
                     }
@@ -547,6 +559,24 @@ namespace rocwmma
                                                 % Traits::MaxKPerIO);
                     }
                 }
+
+                ROCWMMA_DEVICE constexpr static inline auto strideCounts()
+                {
+                    return make_vector((uint32_t)Traits::BlockDimSegs, // BlockDim Segments
+                                       (uint32_t)Traits::BlockKSegs, // BlockK Segments
+                                       (uint32_t)Traits::VWSegs); // VW Segments
+                }
+
+                ROCWMMA_DEVICE constexpr static inline auto strides()
+                {
+                    return make_vector(
+                        make_coord2d((uint32_t)Traits::BlockDimStride_X,
+                                     (uint32_t)Traits::BlockDimStride_Y),
+                        make_coord2d((uint32_t)Traits::BlockKStride_X,
+                                     (uint32_t)Traits::BlockKStride_Y),
+                        make_coord2d((uint32_t)Traits::VWStride_X, (uint32_t)Traits::VWStride_Y));
+                }
+
                 ROCWMMA_DEVICE static inline typename Traits::MatrixCoordT
                     incrementalOffset(uint32_t iteration)
                 {
@@ -626,65 +656,71 @@ namespace rocwmma
             * that VW will be mapped inline with the column.
             * This pattern considers VW up to MaxVW, BlockDim <= 64 and BlockDim > 64.
             *
-            * Iterative thread offset cycles: Fill MaxVW => Fill BlockDim => Fill K
+            * Iterative thread stride cycles (same for all threads):
+            *   Fill MaxVW => Fill BlockK => Fill BlockDim
             *
-            * Index on VW segments first, BlockDimSegs second. Below shows the indexing
-            * order of columns for two full major cycles:
+            * Example:
+            * BlockDim = 256   BlockK = 4
+            * MaxVW = 2       VW = 1
             *
-            * E.g.
-            * WaveSize = 64    Iterations = 16
-            * BlockDim = 256   BlockK = 4          BlockDimSegs = 2
-            * VectorWidth = 1  MaxVectorWidth = 2  VWSegs = 2
+            * BlockDim Stride Count = 4, BlockDimStride = (64, 0)
+            * BlockK   Stride Count = 4, BlockKStride   = (0,  1)
+            * VW       Stride Count = 2, VWStride       = (1,  0)
             *
-            * Minor cycle = VWSegs = 2 iterations
-            * Major cycle = VWSegs * BlockDimSegs = 4 iterations
+            * Stride mapping (BlockDim, BlockK, VW)
+            *  C_n = Matrix column
+            *  i_n = cumulative iteration
             *
-            * iteration offsets:
-            * i0  = (0, 0)   i1  = (1, 0)  i2 = (128, 0) i3 = (129, 0)
-            * i4  = (0, 1)   i5  = (1, 1)  i6 = (128, 1) i7 = (129, 1)
-            * i8  = (0, 2)   i9  = (1, 2)  i10 = (128, 2) i11 = (129, 2)
-            * i12 = (0, 3)  i13  = (1, 3)  i14 = (128, 3) i15 = (129, 3)
+            *  iteration offsets:
+            *  i0  = (0, 0)   i1  = (1, 0)  i2 = (0, 1) i3 = (1, 1)
+            *  i4  = (0, 2)   i5  = (1, 2)  i6 = (0, 3) i7 = (1, 3)
+            *  i8  = (0, 4)   i9  = (1, 4)  i10 = (0, 5) i11 = (1, 5)
+            *  i12 = (0, 6)  i13  = (1, 6)  i14 = (0, 7) i15 = (1, 7)
             *
             *   kDim --------->
             *
-            *   i0    i4    i8    i12
-            *   v_____v_____v_____v_____
-            *   |     |     |     |     |
-            *   i1    i5    i9    i13   |
-            *   v     v     v     v     |
-            *   | C0  | C4  |  C8 | C12 |
-            *   |_____|_____|_____|_____|
-            *   |     |     |     |     |
-            *   |     |     |     |     |
-            *   | C1  | C5  |  C9 | C13 |
-            *   |     |     |     |     |
-            *   i2    i6    i10   i14
-            *   v_____v_____v_____v_____
-            *   |     |     |     |     |
-            *   i3    i7    i11   i15   |
-            *   v     v     v     v     |
-            *   | C2  | C6  | C10 | C14 |
-            *   |     |     |     |     |
-            *   |_____|_____|_____|_____|
-            *   |     |     |     |     |
-            *   |     |     |     |     |
-            *   | C3  | C7  | C11 | C15 |
-            *   |     |     |     |     |
-            *   |_____|_____|_____|_____|
-            *   ^(256, 0)               ^(BlockDim, BlockK)
+            *   BlockDim1
+            *   |                           |-- BlockK Stride = 4 --|
+            *   |                           i0(0,0,0)   i4(0,2,0)
+            *   |            _         _    v_____ _____v_____ _____
+            *   v            |         |    |     |     |     |     |
+            *                |  VW     1    |     |     |     |     |
+            *       BlockDim |  Stride |    | C0  |  C1 |  C2 |  C3 |
+            *        Stride  |         _    v     |     v     |     |
+            *               64              i1(0,0,1)   i5(0,2,1)   |
+            *                |              |     |     |     |     |
+            *                |              |     |     |     |     |
+            *                |              | C0  |  C1 |  C2 |  C3 |
+            *                _              |_____|_____|_____|_____|
+            *                               i8(1,0,0)   i12(1,2,0)
+            *                               v_____ _____v_____ _____
+            *                               |     |     |     |     |
+            *                               |     |     |     |     |
+            *                               | C0  |  C1 |  C2 |  C3 |
+            *                               v     |     v     |     |
+            *                               i9(1,0,1)   i13(1,2,1)  |
+            *                               |     |     |     |     |
+            *                               |     |     |     |     |
+            *                               | C0  |  C1 |  C2 |  C3 |
+            *                               |_____|_____|_____|_____|
+            *                               ...                     ...
+            *                               ...                     ...
+            *                               ^(256, 0)               ^(BlockDim, BlockK)
             *
-            * Register file:
+            * Register file (for all VectorWidths = [MaxVectorWidth, 1]):
             *
-            * Elements 0...........1.............................................64
-            *         ______________________________________________________________
-            * Reg0   |  C0E0  |  C0E2 | ... | C0E62  | C1E0  | C1E2  | ... |  C1E62 |  (MaxVW elements 0 of C0, C1)
-            * Reg1   |  C0E1  |  C0E3 | ... | C0E63  | C1E1  | C1E3  | ... |  C1E63 |  (MaxVW elements 1 of C0, C1)
-            * Reg2   |  C2E0  |  C2E2 | ... | C2E62  | C3E0  | C3E2  | ... |  C3E62 |  (MaxVW elements 0 of C2, C3)
-            * Reg3   |  C2E1  |  C2E3 | ... | C2E63  | C3E1  | C3E3  | ... |  C3E63 |  (MaxVW elements 1 of C2, C3)
-            * Reg4   |  C4E0  |  C4E2 | ... | C4E62  | C5E0  | C5E2  | ... |  C5E62 |  (MaxVW elements 0 of C4, C5)
-            * Reg5   |  C4E1  |  C4E3 | ... | C4E63  | C5E1  | C5E3  | ... |  C5E63 |  (MaxVW elements 1 of C4, C5)
+            * Elements 0...........1........................................... ............64
+            *         ________________________________________________________________________
+            * Reg0   |  C0E0   |  C0E2   | ... |  C0E62   |  C1E0   |  C1E2   | ... |  C1E62  |
+            * Reg1   |  C0E1   |  C0E3   | ... |  C0E63   |  C1E1   |  C1E3   | ... |  C1E63  |
+            * Reg2   |  C2E0   |  C2E2   | ... |  C2E62   |  C3E0   |  C3E2   | ... |  C3E62  |
+            * Reg3   |  C2E1   |  C2E3   | ... |  C2E63   |  C3E1   |  C3E3   | ... |  C3E63  |
+            * Reg4   |  C0E64  |  C0E66  | ... |  C0E126  |  C1E64  |  C1E66  | ... |  C1E126 |
+            * Reg5   |  C0E65  |  C0E67  | ... |  C0E127  |  C1E65  |  C1E67  | ... |  C1E127 |
             * ...      ...
-            * Reg15  |  C14E1 | C14E3 | ... | C14E63 | C15E1 | C15E3 | ... | C15E63 |  (MaxVW elements 1 of C14, C15)
+            * Reg10  |  C2E192 |  C2E194 | ... |  C2E254  |  C3E192 |  C3E194 | ... |  C3E254 |
+            * Reg11  |  C2E193 |  C2E195 | ... |  C2E255  |  C3E193 |  C3E195 | ... |  C3E255 |
+            *
             */
 
             template <uint32_t BlockDim,
@@ -706,16 +742,28 @@ namespace rocwmma
                         MaxElementsPerIO = WaveSize * MaxVectorWidth,
 
                         // Number of BlockDim columns gathered per cycle of MaxVW
-                        MaxKPerIO = MaxElementsPerIO / BlockDim,
+                        MaxKPerIO = std::max(1u, MaxElementsPerIO / BlockDim),
+
+                        VWStride_X = VectorWidth,
+                        VWStride_Y = 0u,
+
+                        BlockDimStride_X = MaxElementsPerIO,
+                        BlockDimStride_Y = 0u,
+
+                        BlockKStride_X = 0u,
+                        BlockKStride_Y = MaxKPerIO,
 
                         // Flag for large BlockDim
                         LargeDim = BlockDim >= MaxElementsPerIO,
 
-                        // Number of column segments (> 0 if LargeDim )
-                        BlockDimSegs = BlockDim / MaxElementsPerIO,
+                        // Number of segments in BlockDim direction
+                        BlockDimSegs = std::max(1u, BlockDim / BlockDimStride_X),
 
-                        // Number of vector width segments
-                        VWSegs = MaxVectorWidth / VectorWidth,
+                        // Number of segments in the BlockK direction
+                        BlockKSegs = std::max(1u, BlockK / BlockKStride_Y),
+
+                        // Number of segments in the MaxVW direction
+                        VWSegs = std::max(1u, MaxVectorWidth / VWStride_X),
 
                         // Log2 Values
                         Log2BlockDim         = Log2<BlockDim>::value,
@@ -736,6 +784,23 @@ namespace rocwmma
                     using MatrixCoordT = Coord2d;
                 };
 
+                ROCWMMA_DEVICE constexpr static inline auto strideCounts()
+                {
+                    return make_vector((uint32_t)Traits::BlockDimSegs, // BlockDim Segments
+                                       (uint32_t)Traits::BlockKSegs, // BlockK Segments
+                                       (uint32_t)Traits::VWSegs); // VW Segments
+                }
+
+                ROCWMMA_DEVICE constexpr static inline auto strides()
+                {
+                    return make_vector(
+                        make_coord2d((uint32_t)Traits::BlockDimStride_X,
+                                     (uint32_t)Traits::BlockDimStride_Y),
+                        make_coord2d((uint32_t)Traits::BlockKStride_X,
+                                     (uint32_t)Traits::BlockKStride_Y),
+                        make_coord2d((uint32_t)Traits::VWStride_X, (uint32_t)Traits::VWStride_Y));
+                }
+
                 ROCWMMA_DEVICE static inline typename Traits::MatrixCoordT baseOffset()
                 {
                     // TODO: Use constexpr if when C++ 17
@@ -859,6 +924,19 @@ namespace rocwmma
                 {
                     return swap(Traits::OrthoLayout::baseOffset());
                 }
+
+                ROCWMMA_DEVICE constexpr static inline auto strideCounts()
+                {
+                    return Traits::OrthoLayout::strideCounts();
+                }
+
+                ROCWMMA_DEVICE constexpr static inline auto strides()
+                {
+                    auto t = Traits::OrthoLayout::strides();
+                    return make_vector(
+                        swap(std::get<0>(t)), swap(std::get<1>(t)), swap(std::get<2>(t)));
+                }
+
                 ROCWMMA_DEVICE static inline typename Traits::MatrixCoordT
                     incrementalOffset(uint32_t iteration)
                 {
@@ -892,6 +970,19 @@ namespace rocwmma
                 {
                     return swap(Traits::OrthoLayout::baseOffset());
                 }
+
+                ROCWMMA_DEVICE constexpr static inline auto strideCounts()
+                {
+                    return Traits::OrthoLayout::strideCounts();
+                }
+
+                ROCWMMA_DEVICE constexpr static inline auto strides()
+                {
+                    auto t = Traits::OrthoLayout::strides();
+                    return make_vector(
+                        swap(std::get<0>(t)), swap(std::get<1>(t)), swap(std::get<2>(t)));
+                }
+
                 ROCWMMA_DEVICE static inline typename Traits::MatrixCoordT
                     incrementalOffset(uint32_t iteration)
                 {
