@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright 2022-2023 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -94,7 +94,7 @@ namespace rocwmma
             {
                 template <typename TT,
                           typename std::enable_if<std::is_integral<TT>{}>::type* = nullptr>
-                ROCWMMA_HOST_DEVICE constexpr static inline auto exec(TT lhs, TT rhs)
+                ROCWMMA_HOST_DEVICE constexpr static inline TT exec(TT lhs, TT rhs)
                 {
                     return lhs & rhs;
                 }
@@ -104,7 +104,7 @@ namespace rocwmma
             {
                 template <typename TT,
                           typename std::enable_if<std::is_integral<TT>{}>::type* = nullptr>
-                ROCWMMA_HOST_DEVICE constexpr static inline auto exec(TT lhs, TT rhs)
+                ROCWMMA_HOST_DEVICE constexpr static inline TT exec(TT lhs, TT rhs)
                 {
                     return lhs | rhs;
                 }
@@ -114,7 +114,7 @@ namespace rocwmma
             {
                 template <typename TT,
                           typename std::enable_if<std::is_integral<TT>{}>::type* = nullptr>
-                ROCWMMA_HOST_DEVICE constexpr static inline auto exec(TT lhs)
+                ROCWMMA_HOST_DEVICE constexpr static inline TT exec(TT lhs)
                 {
                     return ~lhs;
                 }
@@ -124,7 +124,7 @@ namespace rocwmma
             {
                 template <typename TT,
                           typename std::enable_if<std::is_integral<TT>{}>::type* = nullptr>
-                ROCWMMA_HOST_DEVICE constexpr static inline auto exec(TT lhs, TT rhs)
+                ROCWMMA_HOST_DEVICE constexpr static inline TT exec(TT lhs, TT rhs)
                 {
                     return lhs ^ rhs;
                 }
@@ -134,7 +134,7 @@ namespace rocwmma
             {
                 template <typename TT,
                           typename std::enable_if<std::is_integral<TT>{}>::type* = nullptr>
-                ROCWMMA_HOST_DEVICE constexpr static inline auto exec(TT lhs, TT rhs)
+                ROCWMMA_HOST_DEVICE constexpr static inline TT exec(TT lhs, TT rhs)
                 {
                     return lhs >> rhs;
                 }
@@ -144,7 +144,7 @@ namespace rocwmma
             {
                 template <typename TT,
                           typename std::enable_if<std::is_integral<TT>{}>::type* = nullptr>
-                ROCWMMA_HOST_DEVICE constexpr static inline auto exec(TT lhs, TT rhs)
+                ROCWMMA_HOST_DEVICE constexpr static inline TT exec(TT lhs, TT rhs)
                 {
                     return lhs >> rhs;
                 }
@@ -381,7 +381,7 @@ namespace rocwmma
     template <typename... Ts, typename U, typename std::enable_if<(sizeof...(Ts) == Rank)>::type*>
     ROCWMMA_HOST_DEVICE constexpr non_native_vector_base<T, Rank>::non_native_vector_base(
         Ts... args) noexcept
-        : d{args...}
+        : d{static_cast<T>(args)...}
     {
     }
 
@@ -400,35 +400,35 @@ namespace rocwmma
     }
 
     template <typename T, unsigned int Rank>
-    ROCWMMA_HOST_DEVICE inline auto
+    ROCWMMA_HOST_DEVICE constexpr inline auto
         non_native_vector_base<T, Rank>::operator+=(const VecT& x_) noexcept -> VecT&
     {
         return (*this = detail::binOp<detail::ArithmeticOp::Add>(*this, x_, detail::Seq<Rank>{}));
     }
 
     template <typename T, unsigned int Rank>
-    ROCWMMA_HOST_DEVICE inline auto
+    ROCWMMA_HOST_DEVICE constexpr inline auto
         non_native_vector_base<T, Rank>::operator-=(const VecT& x_) noexcept -> VecT&
     {
         return (*this = detail::binOp<detail::ArithmeticOp::Sub>(*this, x_, detail::Seq<Rank>{}));
     }
 
     template <typename T, unsigned int Rank>
-    ROCWMMA_HOST_DEVICE inline auto
+    ROCWMMA_HOST_DEVICE constexpr inline auto
         non_native_vector_base<T, Rank>::operator*=(const VecT& x_) noexcept -> VecT&
     {
         return (*this = detail::binOp<detail::ArithmeticOp::Mult>(*this, x_, detail::Seq<Rank>{}));
     }
 
     template <typename T, unsigned int Rank>
-    ROCWMMA_HOST_DEVICE inline auto
+    ROCWMMA_HOST_DEVICE constexpr inline auto
         non_native_vector_base<T, Rank>::operator/=(const VecT& x_) noexcept -> VecT&
     {
         return (*this = detail::binOp<detail::ArithmeticOp::Div>(*this, x_, detail::Seq<Rank>{}));
     }
 
     template <typename T, unsigned int Rank>
-    ROCWMMA_HOST_DEVICE inline auto
+    ROCWMMA_HOST_DEVICE constexpr inline auto
         non_native_vector_base<T, Rank>::operator+(const VecT& x_) noexcept -> VecT
     {
         auto ret = VecT{*this};
@@ -436,13 +436,29 @@ namespace rocwmma
     }
 
     template <typename T, unsigned int Rank>
-    ROCWMMA_HOST_DEVICE inline auto
+    ROCWMMA_HOST_DEVICE constexpr inline auto
         non_native_vector_base<T, Rank>::operator-(const VecT& x_) noexcept -> VecT
     {
         auto ret = VecT{*this};
         return (ret -= x_);
     }
 
+    template <typename T, unsigned int Rank>
+    ROCWMMA_HOST_DEVICE constexpr inline auto
+        non_native_vector_base<T, Rank>::operator*(const VecT& x_) noexcept -> VecT
+    {
+        auto ret = VecT{*this};
+        return (ret *= x_);
+    }
+
+    template <typename T, unsigned int Rank>
+    ROCWMMA_HOST_DEVICE constexpr inline auto
+        non_native_vector_base<T, Rank>::operator/(const VecT& x_) noexcept -> VecT
+    {
+        auto ret = VecT{*this};
+        return (ret /= x_);
+    }
+
     template <typename T, unsigned int Rank>
     template <typename U, typename std::enable_if<std::is_integral<U>{}>::type*>
     ROCWMMA_HOST_DEVICE inline auto
@@ -459,6 +475,7 @@ namespace rocwmma
         return detail::unOp<detail::ArithmeticOp::Minus>(*this, detail::Seq<Rank>{});
     }
 
+    // @cond
     template <typename T, unsigned int Rank>
     template <typename U, typename std::enable_if<std::is_integral<U>{}>::type*>
     ROCWMMA_HOST_DEVICE inline auto
@@ -466,6 +483,7 @@ namespace rocwmma
     {
         return (*this = detail::binOp<detail::BitwiseOp::And>(*this, x_, detail::Seq<Rank>{}));
     }
+    // @endcond
 
     template <typename T, unsigned int Rank>
     template <typename U, typename std::enable_if<std::is_integral<U>{}>::type*>
@@ -701,38 +719,33 @@ namespace rocwmma
 #define ROCWMMA_REGISTER_HIP_NON_NATIVE_VECTOR_TYPE(TYPE, RANK) \
     ROCWMMA_REGISTER_HIP_VECTOR_BASE(TYPE, RANK, ROCWMMA_HIP_NON_NATIVE_VECTOR_STORAGE_IMPL)
 
-////////////////////////
-/// Bfloat16 support ///
-////////////////////////
-
-// HIP bfloat16_t is not supported in RTC environment
-#if !defined(__HIPCC_RTC__)
+#if defined(__HIPCC_RTC__)
+#define ROCWMMA_VEC_OPERATOR ROCWMMA_DEVICE
+#else
+#define ROCWMMA_VEC_OPERATOR ROCWMMA_HOST_DEVICE
+#endif
 
 // Quirk: explicit specialization for ++ / -- operators in HIP_vector_type<bfloat16_t, N>.
 // Why? bfloat16_t doesn't have automatic conversion from integers so we must override the default implementation;
 // Override such that in(de)crement operators use 1.f instead of 1(int)
-#define ROCWMMA_SPECIALIZE_BFLOAT16_VECTOR_OPERATORS(RANK)                 \
-    template <>                                                            \
-    ROCWMMA_HOST_DEVICE inline HIP_vector_type<rocwmma::bfloat16_t, RANK>& \
-        HIP_vector_type<rocwmma::bfloat16_t, RANK>::operator++() noexcept  \
-    {                                                                      \
-        return *this += HIP_vector_type<rocwmma::bfloat16_t, RANK>{        \
-                   static_cast<rocwmma::bfloat16_t>(1.0f)};                \
-    }                                                                      \
-                                                                           \
-    template <>                                                            \
-    ROCWMMA_HOST_DEVICE inline HIP_vector_type<rocwmma::bfloat16_t, RANK>& \
-        HIP_vector_type<rocwmma::bfloat16_t, RANK>::operator--() noexcept  \
-    {                                                                      \
-        return *this -= HIP_vector_type<rocwmma::bfloat16_t, RANK>{        \
-                   static_cast<rocwmma::bfloat16_t>(1.0f)};                \
+#define ROCWMMA_IMPL_VECTOR_INC_DEC_OPS_AS_FLOAT(FLOAT_TYPE, RANK)                        \
+    template <>                                                                           \
+    ROCWMMA_VEC_OPERATOR inline HIP_vector_type<FLOAT_TYPE, RANK>&                        \
+        HIP_vector_type<FLOAT_TYPE, RANK>::operator++() noexcept                          \
+    {                                                                                     \
+        return *this += HIP_vector_type<FLOAT_TYPE, RANK>{static_cast<FLOAT_TYPE>(1.0f)}; \
+    }                                                                                     \
+                                                                                          \
+    template <>                                                                           \
+    ROCWMMA_VEC_OPERATOR inline HIP_vector_type<FLOAT_TYPE, RANK>&                        \
+        HIP_vector_type<FLOAT_TYPE, RANK>::operator--() noexcept                          \
+    {                                                                                     \
+        return *this -= HIP_vector_type<FLOAT_TYPE, RANK>{static_cast<FLOAT_TYPE>(1.0f)}; \
     }
 
 // Roll the quirk into the registration macro
-#define ROCWMMA_REGISTER_HIP_BFLOAT16_VECTOR_TYPE(RANK)                    \
-    ROCWMMA_REGISTER_HIP_NON_NATIVE_VECTOR_TYPE(rocwmma::bfloat16_t, RANK) \
-    ROCWMMA_SPECIALIZE_BFLOAT16_VECTOR_OPERATORS(RANK)
-
-#endif // !__HIPCC_RTC__
+#define ROCWMMA_REGISTER_HIP_NON_NATIVE_VECTOR_TYPE_WITH_INC_DEC_OPS_AS_FLOAT(TYPE, RANK) \
+    ROCWMMA_REGISTER_HIP_NON_NATIVE_VECTOR_TYPE(TYPE, RANK)                               \
+    ROCWMMA_IMPL_VECTOR_INC_DEC_OPS_AS_FLOAT(TYPE, RANK)
 
 #endif // ROCWMMA_VECTOR_IMPL_HPP
