@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright 2021-2023 Advanced Micro Devices, Inc.
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,10 @@
 #ifndef ROCWMMA_TEST_COMMON_HPP
 #define ROCWMMA_TEST_COMMON_HPP
 
+#if ROCWMMA_TESTS_NO_HALF
+#warning("Building tests with hfloat16_t requires !HIP_NO_HALF && !__HIP_NO_HALF_CONVERSIONS__. Proceeding without hfloat16_t")
+#endif // !ROCWMMA_NO_HALF && __HIP_NO_HALF_CONVERSIONS__
+
 #include <iostream>
 #include <mutex>
 #include <tuple>
@@ -53,18 +57,13 @@
 
 #ifdef ROCWMMA_BENCHMARK_TESTS
 #ifndef CHECK_RSMI_ERROR
-#define CHECK_RSMI_ERROR(expression, smiErrorFlag)                        \
-    if(auto status = (expression); status != RSMI_STATUS_SUCCESS)         \
-    {                                                                     \
-        const char* errName = nullptr;                                    \
-        rsmi_status_string(status, &errName);                             \
-        fprintf(stderr,                                                   \
-                "rsmi error: '%s'(%d) at %s:%d\n",                        \
-                errName,                                                  \
-                status,                                                   \
-                __FILE__,                                                 \
-                __LINE__);                                                \
-        smiErrorFlag = true;                                              \
+#define CHECK_RSMI_ERROR(expression, smiErrorFlag)                                               \
+    if(auto status = (expression); status != RSMI_STATUS_SUCCESS)                                \
+    {                                                                                            \
+        const char* errName = nullptr;                                                           \
+        rsmi_status_string(status, &errName);                                                    \
+        fprintf(stderr, "rsmi error: '%s'(%d) at %s:%d\n", errName, status, __FILE__, __LINE__); \
+        smiErrorFlag = true;                                                                     \
     }
 #endif
 #endif
@@ -100,6 +99,18 @@ namespace rocwmma
         {
         };
 
+#if !defined(ROCBLAS_DATA_TYPE_FLOAT8)
+        template <>
+        struct rocblas_supported<float8_t, float32_t, float32_t> : std::false_type
+        {
+        };
+
+        template <>
+        struct rocblas_supported<bfloat8_t, float32_t, float32_t> : std::false_type
+        {
+        };
+#endif
+
     } // namespace quirks
 
     template <typename Layout>
@@ -138,6 +149,29 @@ namespace rocwmma
             print(mat.data(), m, n, stream);
         }
 
+        template <typename DataT, typename PrintT>
+        __host__ static inline void
+            printAsType(DataT const* mat, uint32_t m, uint32_t n, std::ostream& stream = std::cout)
+        {
+            auto rowMjr = [](uint32_t row, uint32_t col, uint32_t ld) { return row * ld + col; };
+            auto colMjr = [](uint32_t row, uint32_t col, uint32_t ld) { return col * ld + row; };
+
+            auto index = std::is_same<Layout, row_major>::value ? rowMjr : colMjr;
+            auto ld    = std::is_same<Layout, row_major>::value ? n : m;
+
+            for(int i = 0; i < m; ++i) // row
+            {
+                stream << "[ ";
+                for(int j = 0; j < n; ++j) // col
+                {
+                    // (Row, col)
+                    stream << static_cast<PrintT>(mat[index(i, j, ld)]) << " ";
+                }
+                stream << "]\n";
+            }
+            stream << "\n";
+        }
+
         template <typename DataT>
         __host__ static inline void fillWithPadding(
             DataT* mat, uint32_t m, uint32_t n, uint32_t padM, uint32_t padN, DataT padValue)
@@ -287,6 +321,16 @@ namespace rocwmma
             hipLaunchKernelGGL(
                 (fillValKernel<DataT, Layout>), gridDim, blockDim, 0, 0, d_mat, m, n, value);
         }
+
+        // fill kernel wrapper for M x N matrix for mat[i] = i
+        template <typename DataT>
+        __host__ static inline void fillIdxLaunchKernel(DataT* d_mat, uint32_t m, uint32_t n)
+        {
+            auto blockDim = dim3(1024, 1, 1);
+            auto gridDim  = dim3(ceilDiv(m * n, blockDim.x), 1, 1);
+            hipLaunchKernelGGL(
+                (fillIdxKernel<DataT, Layout>), gridDim, blockDim, 0, 0, d_mat, m, n);
+        }
     };
 
     // compareEqual on two different layouts: must calculate index offsets
@@ -376,7 +420,7 @@ namespace rocwmma
         else if(isNaN)
         {
             retval             = false;
-            max_relative_error = std::numeric_limits<TypeA>::signaling_NaN();
+            max_relative_error = double(std::numeric_limits<TypeA>::signaling_NaN());
         }
         else if(max_relative_error > (eps * tolerance))
         {
@@ -465,7 +509,7 @@ namespace rocwmma
         else if(isNaN)
         {
             retval             = false;
-            max_relative_error = std::numeric_limits<TypeA>::signaling_NaN();
+            max_relative_error = double(std::numeric_limits<TypeA>::signaling_NaN());
         }
         else if(max_relative_error > (eps * tolerance))
         {
@@ -588,7 +632,7 @@ namespace rocwmma
         if(isNaN)
         {
             retval           = false;
-            maxRelativeError = std::numeric_limits<TypeA>::signaling_NaN();
+            maxRelativeError = double(std::numeric_limits<TypeA>::signaling_NaN());
         }
         else if(maxRelativeError > (eps * tolerance))
         {
@@ -669,7 +713,7 @@ namespace rocwmma
         if(isNaN)
         {
             retval           = false;
-            maxRelativeError = std::numeric_limits<TypeA>::signaling_NaN();
+            maxRelativeError = double(std::numeric_limits<TypeA>::signaling_NaN());
         }
         else if(maxRelativeError > (eps * tolerance))
         {
